#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from pyasn1_modules import rfc5280
import pyasn1.codec.der.decoder
from pyasn1.type.univ import OctetString
from x509sak.tests.BaseTest import BaseTest
from x509sak.estimate.Validator import ValidationIssue, ValidationJudgement
from x509sak.estimate.GeneralNameValidator import GeneralNameValidator
from x509sak.estimate import JudgementCode

class GeneralNameValidatorTests(BaseTest):
	def _create_general_name(self, name, inner):
		gn = rfc5280.GeneralName()
		if name == "directoryName":
			gn[name] = gn.getComponentByName(name).clone()
			gn[name]["rdnSequence"] = inner
		else:
			gn[name] = gn.getComponentByName(name).clone(inner)
		return gn

	def _validate(self, name, inner, permissible_uri_schemes = None, assert_length = None, assert_present = None, assert_absent = None, additional_errors = None):
		gn = self._create_general_name(name, inner)
		errors = [
			"Enc_DER_Struct_GenName_DirectoryAddress_Empty",
			"Enc_DER_Struct_GenName_DNS_Malformed",
			"Enc_DER_Struct_GenName_DNS_OnlyWhitespace",
			"Enc_DER_Struct_GenName_DNS_SingleLabel",
			"Enc_DER_Struct_GenName_DNS_Wildcard_BroadMatch",
			"Enc_DER_Struct_GenName_DNS_Wildcard_InternationalLabel",
			"Enc_DER_Struct_GenName_DNS_Wildcard_MulitpleWildcards",
			"Enc_DER_Struct_GenName_DNS_Wildcard_NotLeftmost",
			"Enc_DER_Struct_GenName_DNS_Wildcard_NotPermitted",
			"Enc_DER_Struct_GenName_Email_Malformed",
			"Enc_DER_Struct_GenName_IPAddress_Malformed",
			"Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace",
			"Enc_DER_Struct_GenName_URI_LDAP_NoAttrdesc",
			"Enc_DER_Struct_GenName_URI_LDAP_NoDN",
			"Enc_DER_Struct_GenName_URI_LDAP_NoHostname",
			"Enc_DER_Struct_GenName_URI_Malformed",
			"Enc_DER_Struct_GenName_URI_UncommonURIScheme",
		]
		valid_errors = set([
			"Enc_DER_Struct_GenName_DirectoryAddress_Unexpected",
			"Enc_DER_Struct_GenName_DNS_Unexpected",
			"Enc_DER_Struct_GenName_Email_Unexpected",
			"Enc_DER_Struct_GenName_IPAddress_Unexpected",
			"Enc_DER_Struct_GenName_URI_Unexpected",
		])
		if additional_errors is not None:
			self.assertTrue(all(error in valid_errors for error in additional_errors))
			errors += additional_errors
		recognized_issues = { name: ValidationIssue(code = JudgementCode.X509sakIssues_AnalysisNotImplemented, judgement = ValidationJudgement(info_payload = name)) for name in errors }
		result = GeneralNameValidator("Test GN", recognized_issues, permissible_uri_schemes = permissible_uri_schemes).validate_asn1(gn)
		if assert_length is not None:
			self.assertEqual(len(result), assert_length)
		if assert_present is not None:
			code_set = set(judgement.info_payload for judgement in result)
			self.assertIn(assert_present, code_set)
		if assert_absent is not None:
			code_set = set(judgement.info_payload for judgement in result)
			self.assertNotIn(assert_absent, code_set)
		return result

	def test_email_ok(self):
		self._validate("rfc822Name", "foo@bar.com", assert_length = 0)

	def test_email_bad(self):
		self._validate("rfc822Name", "foo @bar.com", assert_present = "Enc_DER_Struct_GenName_Email_Malformed")

	def test_ipv4_ok(self):
		self._validate("iPAddress", OctetString(bytes.fromhex("aa bb cc dd")), assert_length = 0)

	def test_ipv6_ok(self):
		self._validate("iPAddress", OctetString(bytes(16)), assert_length = 0)

	def test_ip_bad(self):
		self._validate("iPAddress", OctetString(bytes(2)), assert_present = "Enc_DER_Struct_GenName_IPAddress_Malformed")

	def test_ip_private_class_a(self):
		self._validate("iPAddress", OctetString(bytes([ 10, 42, 117, 83 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_ip_private_class_b(self):
		self._validate("iPAddress", OctetString(bytes([ 172, 31, 12, 4 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_ip_private_class_c(self):
		self._validate("iPAddress", OctetString(bytes([ 192, 168, 17, 44 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_ip_multicast(self):
		self._validate("iPAddress", OctetString(bytes([ 239, 255, 255, 255 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_ip_broadcast(self):
		self._validate("iPAddress", OctetString(bytes([ 255, 255, 255, 255 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_ip_loopback(self):
		self._validate("iPAddress", OctetString(bytes([ 127, 255, 255, 255 ])), assert_present = "Enc_DER_Struct_GenName_IPAddress_PrivateAddressSpace")

	def test_dnsname_ok(self):
		self._validate("dNSName", "foobar.com", assert_length = 0)

	def test_dnsname_single_label(self):
		self._validate("dNSName", "com", assert_present = "Enc_DER_Struct_GenName_DNS_SingleLabel")

	def test_dnsname_space(self):
		self._validate("dNSName", " ", assert_present = "Enc_DER_Struct_GenName_DNS_OnlyWhitespace")

	def test_dnsname_bad1(self):
		self._validate("dNSName", "muh kuh.com", assert_present = "Enc_DER_Struct_GenName_DNS_Malformed")

	def test_dnsname_bad2(self):
		self._validate("dNSName", "muh\xffkuh.com", assert_present = "Enc_DER_Struct_GenName_DNS_Malformed")

	def test_dnsname_bad3(self):
		self._validate("dNSName", "muh\x00kuh.com", assert_present = "Enc_DER_Struct_GenName_DNS_Malformed")

	def test_dnsname_bad4(self):
		self._validate("dNSName", "", assert_present = "Enc_DER_Struct_GenName_DNS_Malformed")

	def test_uri_ok(self):
		self._validate("uniformResourceIdentifier", "http://google.com", assert_length = 0)
		self._validate("uniformResourceIdentifier", "http://google.com/", assert_length = 0)
		self._validate("uniformResourceIdentifier", "http://google.com/foo.crt", assert_length = 0)

	def test_uri_bad1(self):
		self._validate("uniformResourceIdentifier", "http:/google.com/foo.crt", assert_present = "Enc_DER_Struct_GenName_URI_Malformed")

	def test_uri_bad2(self):
		self._validate("uniformResourceIdentifier", "ldap://google.com/foo.crt", permissible_uri_schemes = [ "http", "https", "ldap" ], assert_length = 0)
		self._validate("uniformResourceIdentifier", "ldap://google.com/foo.crt", permissible_uri_schemes = [ "http", "https" ], assert_present = "Enc_DER_Struct_GenName_URI_UncommonURIScheme")

	def test_uri_unexpected(self):
		errname = "Enc_DER_Struct_GenName_URI_Unexpected"
		self._validate("uniformResourceIdentifier", "http://foo.com", assert_absent = errname)
		self._validate("uniformResourceIdentifier", "http://foo.com", additional_errors = [ errname ], assert_present = errname)

	def test_ip_unexpected(self):
		errname = "Enc_DER_Struct_GenName_IPAddress_Unexpected"
		self._validate("iPAddress", OctetString(bytes([ 172, 31, 12, 4 ])), assert_absent = errname)
		self._validate("iPAddress", OctetString(bytes([ 172, 31, 12, 4 ])), additional_errors = [ errname ], assert_present = errname)

	def test_dns_unexpected(self):
		errname = "Enc_DER_Struct_GenName_DNS_Unexpected"
		self._validate("dNSName", "foo.bar.com", assert_absent = errname)
		self._validate("dNSName", "foo.bar.com", additional_errors = [ errname ], assert_present = errname)

	def test_email_unexpected(self):
		errname = "Enc_DER_Struct_GenName_Email_Unexpected"
		self._validate("rfc822Name", "foo@bar.com", assert_absent = errname)
		self._validate("rfc822Name", "foo@bar.com", additional_errors = [ errname ], assert_present = errname)

	def test_directoryaddress_unexpected(self):
		errname = "Enc_DER_Struct_GenName_DirectoryAddress_Unexpected"
		dirname_data = bytes.fromhex("a4 1a 30 18 31 16 30 14 06 03 55 04 03 13 0d 64 69 72 65 63 74 6f 72 79 4e 61 6d 65")
		(dirname, _) = pyasn1.codec.der.decoder.decode(dirname_data)
		self._validate("directoryName", dirname, assert_absent = errname)
		self._validate("directoryName", dirname, additional_errors = [ errname ], assert_present = errname)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import tempfile
import json
from x509sak.tests import BaseTest, ResourceFileLoader
from x509sak.CertificateAnalyzer import CertificateAnalyzer

class CmdLineTestsExamine(BaseTest):
	def test_crt_with_custom_key_usage(self):
		with ResourceFileLoader("certs/ok/custom_key_usage.pem") as certfile:
			output = self._run_x509sak([ "examine", certfile ]).stdout
			self.assertIn(b"CN = 0b239049", output)
			self.assertIn(b"CN = \"Root CA\"", output)
			self.assertIn(b"ECC on prime256v1", output)

	def test_examine_write_json(self):
		with ResourceFileLoader("certs/ok/custom_key_usage.pem") as crtfile, tempfile.NamedTemporaryFile(prefix = "crt_", suffix = ".json") as jsonfile:
			self._run_x509sak([ "examine", "-f", "json", "-o", jsonfile.name, crtfile ])
			with open(jsonfile.name) as jsonfile:
				json_data = json.load(jsonfile)
			self.assertEqual(json_data["data"][0]["issuer"]["rfc2253"], "CN=Root CA")
			self.assertEqual(json_data["data"][0]["subject"]["rfc2253"], "CN=0b239049-3d65-46c2-8fdd-90f13cadc70b")
			self.assertEqual(json_data["data"][0]["validity"]["not_before"]["iso"], "2018-07-14T16:00:53Z")
			self.assertEqual(json_data["data"][0]["validity"]["not_after"]["iso"], "2019-07-14T16:00:53Z")

	def test_purpose_ca(self):
		with ResourceFileLoader("certs/ok/johannes-bauer-root.pem") as crtfile:
			self._run_x509sak([ "examine", "-p", "ca", "--fast-rsa", crtfile ])

	def test_purpose_tls_server(self):
		with ResourceFileLoader("certs/ok/johannes-bauer.com.pem") as crtfile:
			output = self._run_x509sak([ "examine", "-p", "tls-server", "-n", "johannes-bauer.com", crtfile ]).stdout_text
			self.assertIn("Subject Alternative Name matches 'johannes-bauer.com'", output)

	def test_encodings(self):
		with ResourceFileLoader("certs/ok/johannes-bauer-intermediate.pem") as crtfile:
			self._run_x509sak([ "examine", "-p", "ca", "--fast-rsa", "-f", "ansitext", crtfile ])
			self._run_x509sak([ "examine", "-p", "ca", "--fast-rsa", "-f", "text", crtfile ])
			self._run_x509sak([ "examine", "-p", "ca", "--fast-rsa", "-f", "json", crtfile ])

	def test_rsa_pss_default(self):
		with ResourceFileLoader("certs/ok/rsapss_defaults.pem") as crtfile:
			self._run_x509sak([ "examine", "-p", "tls-client", "--fast-rsa", "-f", "json", crtfile ]).stdout_json

	def test_rsa_pss_custom(self):
		with ResourceFileLoader("certs/ok/rsapss_sha256_salt_32.pem") as crtfile:
			self._run_x509sak([ "examine", "-p", "tls-client", "--fast-rsa", "-f", "json", crtfile ]).stdout_json

	def _test_examine_x509test_resultcode(self, certname, expect_present = None, expect_absent = None, parent_certname = None, fast_rsa = True, host_check = None, include_raw = False, purpose = None):
		if expect_present is None:
			expect_present = tuple()
		if not isinstance(expect_present, (list, tuple)):
			expect_present = (expect_present, )

		if expect_absent is None:
			expect_absent = tuple()
		if not isinstance(expect_absent, (list, tuple)):
			expect_absent = (expect_absent, )

		def gen_cmdline(fast_rsa, host_check, include_raw, certfile_name, cacertfile_name, outfile_name, outformat = "json"):
			cmdline = [ "examine" ]
			if fast_rsa:
				cmdline += [ "--fast-rsa" ]
			cmdline += [ "-f", outformat ]
			cmdline += [ "-o", outfile_name ]
			if cacertfile_name is not None:
				cmdline += [ "--parent-certificate", cacertfile_name ]
			if host_check is not None:
				cmdline += [ "-p", "tls-server", "--server-name", host_check ]
			elif purpose is not None:
				cmdline += [ "-p", purpose ]

			if include_raw:
				cmdline += [ "--include-raw-data" ]
			cmdline += [ certfile_name ]
			return cmdline

		with ResourceFileLoader(certname) as certfile, tempfile.NamedTemporaryFile(suffix = ".json") as outfile:
			if parent_certname is None:
				cmdline = gen_cmdline(fast_rsa, host_check, include_raw, certfile_name = certfile, cacertfile_name = None, outfile_name = outfile.name, outformat = "ansitext")
				self._run_x509sak(cmdline)
				cmdline = gen_cmdline(fast_rsa, host_check, include_raw, certfile_name = certfile, cacertfile_name = None, outfile_name = outfile.name, outformat = "json")
				self._run_x509sak(cmdline)
			else:
				with ResourceFileLoader(parent_certname) as parent_crt:
					cmdline = gen_cmdline(fast_rsa, host_check, include_raw, certfile_name = certfile, cacertfile_name = parent_crt, outfile_name = outfile.name, outformat = "ansitext")
					self._run_x509sak(cmdline)
					cmdline = gen_cmdline(fast_rsa, host_check, include_raw, certfile_name = certfile, cacertfile_name = parent_crt, outfile_name = outfile.name, outformat = "json")
					self._run_x509sak(cmdline)

			# Read all codes from the generated JSON
			with open(outfile.name) as f:
				data = json.load(f)
			encountered_codes = CertificateAnalyzer.extract_codes_from_json(data)

			for code in expect_present:
				self.assertIn(code, encountered_codes)
			for code in expect_absent:
				self.assertNotIn(code, encountered_codes)

	def test_examine_x509test_xf_algo_mismatch1(self):
		self._test_examine_x509test_resultcode("certs/x509test/xf-algo-mismatch1.pem", "X509Cert_Signature_Function_BodyMismatch")

	def test_examine_x509test_xf_ext_auth_keyid_mismatch(self):
		self._test_examine_x509test_resultcode("certs/x509test/xf-ext-auth-keyid-mismatch.pem", "CertUsage_CARelationship_AKI_KeyID_Mismatch", parent_certname = "certs/x509test/ok-ca.pem")

	def test_hostname_ok(self):
		self._test_examine_x509test_resultcode("certs/ok/johannes-bauer.com.pem", "CertUsage_Purpose_ServerCert_SAN_Match", expect_absent = "CertUsage_Purpose_ServerCert_SAN_Mismatch", host_check = "mail.johannes-bauer.com")

	def test_constructed_rsa_modulus_prime(self):
		self._test_examine_x509test_resultcode("certs/constructed/rsa_modulus_prime.pem", "X509Cert_PublicKey_RSA_Modulus_Prime", fast_rsa = False)

	def test_include_raw_data_ecc_f2m(self):
		self._test_examine_x509test_resultcode("certs/ok/ecc_sect283r1.pem", include_raw = True)

	def test_check_no_ca_when_expecting_ca(self):
		self._test_examine_x509test_resultcode("certs/ok/johannes-bauer.com.pem", expect_present = "CertUsage_Purpose_CACert_NoCACert", purpose = "ca")



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import time
import hashlib

class PRNG():
	def __init__(self, seed = None):
		self._seed = seed
		self._buffer = bytearray()
		self.reset()

	def get(self, length):
		self._fill_buffer(length)
		(result, self._buffer) = (self._buffer[ : length], self._buffer[length : ])
		return result

	def _fill_buffer(self, length):
		while len(self._buffer) < length:
			self._buffer += self._next_chunk()

	def reset(self):
		self.init_state(self._seed or b"123456789")

	def init_state(self, state):
		raise NotImplementedError("init_state")

	def _next_chunk(self):
		raise NotImplementedError("_next_chunk")

	def profile(self, min_time_secs = 3):
		t0 = time.time()
		length = 0
		while True:
			data = self.get(128 * 1024)
			length += len(data)
			t1 = time.time()
			tdiff = t1 - t0
			if tdiff > min_time_secs:
				break
		print("%d bytes in %.1f secs: %.1f MBytes/sec" % (length, tdiff, length / tdiff / 1e6))

	def write_fp(self, fp, length):
		remaining = length
		while remaining > 0:
			chunk_size = min(1024 * 1024, remaining)
			data = self.get(chunk_size)
			fp.write(data)
			remaining -= len(data)

	def write_file(self, filename, length):
		with open(filename, "wb") as f:
			self.write_fp(f, length)

	def write_bracketed(self, filename, prefix_len, data, suffix_len):
		with open(filename, "wb") as f:
			self.write_fp(f, prefix_len)
			f.write(data)
			self.write_fp(f, suffix_len)

class HashedPRNG(PRNG):
	"""This PRNG is *not* a CSPRNG. It's intended purely for testing
	purposes."""

	_HASHFNC = hashlib.sha512

	def init_state(self, state):
		self._state = self._HASHFNC(state).digest()

	def _next_chunk(self):
		result = self._state
		self._state = self._HASHFNC(self._state).digest()
		return result



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.tests import BaseAnalyzerTest

class GeneratedSecurityAnalyzerTests(BaseAnalyzerTest):
# ====================================================================== Begin of malencoding ======================================================================
	def test_generated_malencoding_ext_AIA_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AIA_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AIA_Malformed_Undecodable")

	def test_generated_malencoding_ext_AIA_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AIA_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AIA_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_AIA_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AIA_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AIA_TrailingData")

	def test_generated_malencoding_ext_AIA_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AIA_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AIA_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_AKI_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AKI_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_Malformed_Undecodable")

	def test_generated_malencoding_ext_AKI_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AKI_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_AKI_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AKI_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_TrailingData")

	def test_generated_malencoding_ext_AKI_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_AKI_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_BC_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_BC_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_BC_Malformed_Undecodable")

	def test_generated_malencoding_ext_BC_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_BC_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_BC_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_BC_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_BC_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_BC_TrailingData")

	def test_generated_malencoding_ext_BC_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_BC_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_BC_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_CP_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CP_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CP_Malformed_Undecodable")

	def test_generated_malencoding_ext_CP_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CP_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CP_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_CP_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CP_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CP_TrailingData")

	def test_generated_malencoding_ext_CP_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CP_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CP_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_CRLDP_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CRLDP_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_Malformed_Undecodable")

	def test_generated_malencoding_ext_CRLDP_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CRLDP_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_CRLDP_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CRLDP_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_TrailingData")

	def test_generated_malencoding_ext_CRLDP_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_CRLDP_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_EKU_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_EKU_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_EKU_Malformed_Undecodable")

	def test_generated_malencoding_ext_EKU_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_EKU_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_EKU_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_EKU_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_EKU_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_EKU_TrailingData")

	def test_generated_malencoding_ext_EKU_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_EKU_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_EKU_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_FCRL_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_FCRL_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_FCRL_Malformed_Undecodable")

	def test_generated_malencoding_ext_FCRL_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_FCRL_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_FCRL_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_FCRL_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_FCRL_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_FCRL_TrailingData")

	def test_generated_malencoding_ext_FCRL_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_FCRL_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_FCRL_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_IAN_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAN_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Malformed_Undecodable")

	def test_generated_malencoding_ext_IAN_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAN_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_IAN_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAN_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_TrailingData")

	def test_generated_malencoding_ext_IAN_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAN_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_IAP_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAP_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAP_Malformed_Undecodable")

	def test_generated_malencoding_ext_IAP_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAP_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAP_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_IAP_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAP_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAP_TrailingData")

	def test_generated_malencoding_ext_IAP_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_IAP_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAP_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_KU_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_KU_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_KU_Malformed_Undecodable")

	def test_generated_malencoding_ext_KU_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_KU_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_KU_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_KU_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_KU_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_KU_TrailingData")

	def test_generated_malencoding_ext_KU_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_KU_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_KU_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_NC_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NC_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_Malformed_Undecodable")

	def test_generated_malencoding_ext_NC_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NC_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_NC_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NC_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_TrailingData")

	def test_generated_malencoding_ext_NC_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NC_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_NSCT_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NSCT_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NSCT_Malformed_Undecodable")

	def test_generated_malencoding_ext_NSCT_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NSCT_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NSCT_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_NSCT_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NSCT_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NSCT_TrailingData")

	def test_generated_malencoding_ext_NSCT_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_NSCT_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NSCT_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_PC_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PC_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PC_Malformed_Undecodable")

	def test_generated_malencoding_ext_PC_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PC_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PC_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_PC_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PC_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PC_TrailingData")

	def test_generated_malencoding_ext_PC_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PC_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PC_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_PM_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PM_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PM_Malformed_Undecodable")

	def test_generated_malencoding_ext_PM_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PM_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PM_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_PM_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PM_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PM_TrailingData")

	def test_generated_malencoding_ext_PM_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_PM_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_PM_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_SAN_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SAN_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Malformed_Undecodable")

	def test_generated_malencoding_ext_SAN_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SAN_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_SAN_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SAN_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_TrailingData")

	def test_generated_malencoding_ext_SAN_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SAN_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_SDA_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SDA_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SDA_Malformed_Undecodable")

	def test_generated_malencoding_ext_SDA_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SDA_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SDA_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_SDA_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SDA_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SDA_TrailingData")

	def test_generated_malencoding_ext_SDA_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SDA_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SDA_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_SIA_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SIA_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SIA_Malformed_Undecodable")

	def test_generated_malencoding_ext_SIA_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SIA_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SIA_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_SIA_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SIA_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SIA_TrailingData")

	def test_generated_malencoding_ext_SIA_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SIA_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SIA_Malformed_UnexpectedType")

	def test_generated_malencoding_ext_SKI_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SKI_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SKI_Malformed_Undecodable")

	def test_generated_malencoding_ext_SKI_non_der(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SKI_non_der.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SKI_Malformed_NonDEREncoding")

	def test_generated_malencoding_ext_SKI_trailing_data(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SKI_trailing_data.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SKI_TrailingData")

	def test_generated_malencoding_ext_SKI_wrong_type(self):
		self._test_examine_x509test_resultcode("certs/generated/malencoding/ext_SKI_wrong_type.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SKI_Malformed_UnexpectedType")

# ====================================================================== End of malencoding ======================================================================

# ====================================================================== Begin of generalname ======================================================================
	def test_generated_generalname_ext_AKI_caname_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DirectoryAddress_Empty")

	def test_generated_generalname_ext_AKI_caname_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DNS_Malformed")

	def test_generated_generalname_ext_AKI_caname_dns_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dns_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DNS_Unexpected")

	def test_generated_generalname_ext_AKI_caname_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DNS_SingleLabel")

	def test_generated_generalname_ext_AKI_caname_dns_wc_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dns_wc_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DNS_Wildcard_NotPermitted")

	def test_generated_generalname_ext_AKI_caname_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_AKI_caname_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_AKI_caname_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_Email_Malformed")

	def test_generated_generalname_ext_AKI_caname_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_Email_Unexpected")

	def test_generated_generalname_ext_AKI_caname_ip_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_ip_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_Malformed")

	def test_generated_generalname_ext_AKI_caname_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_Unexpected")

	def test_generated_generalname_ext_AKI_caname_ip_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_ip_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_AKI_caname_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_Unexpected")

	def test_generated_generalname_ext_AKI_caname_ipv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_ipv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_AKI_caname_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_OtherName_Unexpected")

	def test_generated_generalname_ext_AKI_caname_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_RegisteredID_Unexpected")

	def test_generated_generalname_ext_AKI_caname_subnet_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_subnet_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_Malformed")

	def test_generated_generalname_ext_AKI_caname_subnetv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_subnetv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_IPAddress_Malformed")

	def test_generated_generalname_ext_AKI_caname_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_URI_Malformed")

	def test_generated_generalname_ext_AKI_caname_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_URI_Unexpected")

	def test_generated_generalname_ext_AKI_caname_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_URI_UncommonURIScheme")

	def test_generated_generalname_ext_AKI_caname_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_X400Address_Unexpected")

	def test_generated_generalname_ext_AKI_caname_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_AKI_caname_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_AKI_CAName_X400Address_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DirectoryAddress_Empty")

	def test_generated_generalname_ext_CRLDP_issuer_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DNS_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_dns_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dns_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DNS_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DNS_SingleLabel")

	def test_generated_generalname_ext_CRLDP_issuer_dns_wc_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dns_wc_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DNS_Wildcard_NotPermitted")

	def test_generated_generalname_ext_CRLDP_issuer_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_CRLDP_issuer_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_Email_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_Email_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_ip_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_ip_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_ip_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_ip_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_CRLDP_issuer_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_ipv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_ipv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_CRLDP_issuer_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_OtherName_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_RegisteredID_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_subnet_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_subnet_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_subnetv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_subnetv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_URI_Malformed")

	def test_generated_generalname_ext_CRLDP_issuer_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_URI_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_URI_UncommonURIScheme")

	def test_generated_generalname_ext_CRLDP_issuer_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_X400Address_Unexpected")

	def test_generated_generalname_ext_CRLDP_issuer_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_issuer_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_CRLIssuer_X400Address_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DirectoryAddress_Empty")

	def test_generated_generalname_ext_CRLDP_point_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DNS_Malformed")

	def test_generated_generalname_ext_CRLDP_point_dns_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dns_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DNS_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DNS_SingleLabel")

	def test_generated_generalname_ext_CRLDP_point_dns_wc_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dns_wc_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DNS_Wildcard_NotPermitted")

	def test_generated_generalname_ext_CRLDP_point_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_CRLDP_point_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_Email_Malformed")

	def test_generated_generalname_ext_CRLDP_point_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_Email_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_ip_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_ip_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_point_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_ip_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_ip_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_CRLDP_point_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_ipv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_ipv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_CRLDP_point_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_OtherName_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_RegisteredID_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_subnet_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_subnet_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_point_subnetv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_subnetv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_IPAddress_Malformed")

	def test_generated_generalname_ext_CRLDP_point_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_URI_Malformed")

	def test_generated_generalname_ext_CRLDP_point_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_URI_UncommonURIScheme")

	def test_generated_generalname_ext_CRLDP_point_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_X400Address_Unexpected")

	def test_generated_generalname_ext_CRLDP_point_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_CRLDP_point_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_CRLDP_PointName_X400Address_Unexpected")

	def test_generated_generalname_ext_IAN_name_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DirectoryAddress_Empty")

	def test_generated_generalname_ext_IAN_name_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DNS_Malformed")

	def test_generated_generalname_ext_IAN_name_dns_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dns_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DNS_Unexpected")

	def test_generated_generalname_ext_IAN_name_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DNS_SingleLabel")

	def test_generated_generalname_ext_IAN_name_dns_wc_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dns_wc_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DNS_Wildcard_NotPermitted")

	def test_generated_generalname_ext_IAN_name_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_IAN_name_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_IAN_name_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_Email_Malformed")

	def test_generated_generalname_ext_IAN_name_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_Email_Unexpected")

	def test_generated_generalname_ext_IAN_name_ip_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_ip_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_IAN_name_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_Unexpected")

	def test_generated_generalname_ext_IAN_name_ip_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_ip_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_IAN_name_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_Unexpected")

	def test_generated_generalname_ext_IAN_name_ipv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_ipv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_IAN_name_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_OtherName_Unexpected")

	def test_generated_generalname_ext_IAN_name_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_RegisteredID_Unexpected")

	def test_generated_generalname_ext_IAN_name_subnet_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_subnet_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_IAN_name_subnetv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_subnetv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_IAN_name_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_URI_Malformed")

	def test_generated_generalname_ext_IAN_name_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_URI_Unexpected")

	def test_generated_generalname_ext_IAN_name_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_URI_UncommonURIScheme")

	def test_generated_generalname_ext_IAN_name_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_IAN_name_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_IAN_name_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_IAN_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_NC_excluded_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DirectoryAddress_Empty")

	def test_generated_generalname_ext_NC_excluded_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_Malformed")

	def test_generated_generalname_ext_NC_excluded_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_SingleLabel")

	def test_generated_generalname_ext_NC_excluded_dns_wc_broad(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_wc_broad.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_Wildcard_BroadMatch")

	def test_generated_generalname_ext_NC_excluded_dns_wc_international(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_wc_international.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_Wildcard_InternationalLabel")

	def test_generated_generalname_ext_NC_excluded_dns_wc_multiple(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_wc_multiple.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_Wildcard_MulitpleWildcards")

	def test_generated_generalname_ext_NC_excluded_dns_wc_notleftmost(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_wc_notleftmost.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_Wildcard_NotLeftmost")

	def test_generated_generalname_ext_NC_excluded_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_NC_excluded_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_NC_excluded_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_Email_Malformed")

	def test_generated_generalname_ext_NC_excluded_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_Email_Unexpected")

	def test_generated_generalname_ext_NC_excluded_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_NC_excluded_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_NC_excluded_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_OtherName_Unexpected")

	def test_generated_generalname_ext_NC_excluded_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_RegisteredID_Unexpected")

	def test_generated_generalname_ext_NC_excluded_subnet_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_subnet_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_MalformedSubnet")

	def test_generated_generalname_ext_NC_excluded_subnet_noncidr(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_subnet_noncidr.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_NonCIDRBlock")

	def test_generated_generalname_ext_NC_excluded_subnet_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_subnet_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_NC_excluded_subnetv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_subnetv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_NC_excluded_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_URI_Malformed")

	def test_generated_generalname_ext_NC_excluded_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_URI_Unexpected")

	def test_generated_generalname_ext_NC_excluded_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_URI_UncommonURIScheme")

	def test_generated_generalname_ext_NC_excluded_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_NC_excluded_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_excluded_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_ExcludedSubtree_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_NC_permitted_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DirectoryAddress_Empty")

	def test_generated_generalname_ext_NC_permitted_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_Malformed")

	def test_generated_generalname_ext_NC_permitted_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_SingleLabel")

	def test_generated_generalname_ext_NC_permitted_dns_wc_broad(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_wc_broad.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_Wildcard_BroadMatch")

	def test_generated_generalname_ext_NC_permitted_dns_wc_international(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_wc_international.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_Wildcard_InternationalLabel")

	def test_generated_generalname_ext_NC_permitted_dns_wc_multiple(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_wc_multiple.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_Wildcard_MulitpleWildcards")

	def test_generated_generalname_ext_NC_permitted_dns_wc_notleftmost(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_wc_notleftmost.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_Wildcard_NotLeftmost")

	def test_generated_generalname_ext_NC_permitted_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_NC_permitted_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_NC_permitted_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_Email_Malformed")

	def test_generated_generalname_ext_NC_permitted_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_Email_Unexpected")

	def test_generated_generalname_ext_NC_permitted_ip_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_ip_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_NC_permitted_ipv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_ipv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_NC_permitted_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_OtherName_Unexpected")

	def test_generated_generalname_ext_NC_permitted_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_RegisteredID_Unexpected")

	def test_generated_generalname_ext_NC_permitted_subnet_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_subnet_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_MalformedSubnet")

	def test_generated_generalname_ext_NC_permitted_subnet_noncidr(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_subnet_noncidr.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_NonCIDRBlock")

	def test_generated_generalname_ext_NC_permitted_subnet_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_subnet_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_NC_permitted_subnetv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_subnetv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_NC_permitted_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_URI_Malformed")

	def test_generated_generalname_ext_NC_permitted_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_URI_Unexpected")

	def test_generated_generalname_ext_NC_permitted_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_URI_UncommonURIScheme")

	def test_generated_generalname_ext_NC_permitted_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_NC_permitted_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_NC_permitted_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_NC_PermittedSubtree_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_SAN_name_dirname_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dirname_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DirectoryAddress_Empty")

	def test_generated_generalname_ext_SAN_name_dirname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dirname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DirectoryAddress_Unexpected")

	def test_generated_generalname_ext_SAN_name_dns_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_Malformed")

	def test_generated_generalname_ext_SAN_name_dns_single_label(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_single_label.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_SingleLabel")

	def test_generated_generalname_ext_SAN_name_dns_wc_broad(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_wc_broad.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_Wildcard_BroadMatch")

	def test_generated_generalname_ext_SAN_name_dns_wc_international(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_wc_international.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_Wildcard_InternationalLabel")

	def test_generated_generalname_ext_SAN_name_dns_wc_multiple(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_wc_multiple.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_Wildcard_MulitpleWildcards")

	def test_generated_generalname_ext_SAN_name_dns_wc_notleftmost(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_wc_notleftmost.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_Wildcard_NotLeftmost")

	def test_generated_generalname_ext_SAN_name_dns_whitespace(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_dns_whitespace.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_DNS_OnlyWhitespace")

	def test_generated_generalname_ext_SAN_name_edipartyname_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_edipartyname_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_EDIPartyName_Unexpected")

	def test_generated_generalname_ext_SAN_name_email_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_email_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_Email_Malformed")

	def test_generated_generalname_ext_SAN_name_email_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_email_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_Email_Unexpected")

	def test_generated_generalname_ext_SAN_name_ip_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_ip_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_SAN_name_ip_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_ip_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_SAN_name_ipv6_private(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_ipv6_private.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_IPAddress_PrivateAddressSpace")

	def test_generated_generalname_ext_SAN_name_othername_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_othername_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_OtherName_Unexpected")

	def test_generated_generalname_ext_SAN_name_registeredid_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_registeredid_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_RegisteredID_Unexpected")

	def test_generated_generalname_ext_SAN_name_subnet_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_subnet_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_SAN_name_subnetv6_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_subnetv6_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_IPAddress_Malformed")

	def test_generated_generalname_ext_SAN_name_uri_malformed(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_uri_malformed.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_URI_Malformed")

	def test_generated_generalname_ext_SAN_name_uri_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_uri_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_URI_Unexpected")

	def test_generated_generalname_ext_SAN_name_uri_uncommon_scheme(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_uri_uncommon_scheme.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_URI_UncommonURIScheme")

	def test_generated_generalname_ext_SAN_name_x400address_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_x400address_empty.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_X400Address_Unexpected")

	def test_generated_generalname_ext_SAN_name_x400address_ok(self):
		self._test_examine_x509test_resultcode("certs/generated/generalname/ext_SAN_name_x400address_ok.pem", expect_present = "X509Cert_Body_X509Exts_Ext_SAN_Name_X400Address_Unexpected")

# ====================================================================== End of generalname ======================================================================

# ====================================================================== Begin of distinguishedname ======================================================================
	def test_generated_distinguishedname_dn_issuer_duplicate_rdns(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_duplicate_rdns.pem", expect_present = "X509Cert_Body_Issuer_DuplicateRDNs")

	def test_generated_distinguishedname_dn_issuer_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_empty.pem", expect_present = "X509Cert_Body_Issuer_Empty")

	def test_generated_distinguishedname_dn_issuer_multiple_cn(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_multiple_cn.pem", expect_present = "X509Cert_Body_Issuer_MultipleCN")

	def test_generated_distinguishedname_dn_issuer_no_cn(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_no_cn.pem", expect_present = "X509Cert_Body_Issuer_NoCN")

	def test_generated_distinguishedname_dn_issuer_rdn_deprecated_type(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_deprecated_type.pem", expect_present = "X509Cert_Body_Issuer_RDN_DeprecatedType")

	def test_generated_distinguishedname_dn_issuer_rdn_duplicate_key(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_duplicate_key.pem", expect_present = "X509Cert_Body_Issuer_RDN_DuplicateSet_Key")

	def test_generated_distinguishedname_dn_issuer_rdn_duplicate_key_value(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_duplicate_key_value.pem", expect_present = "X509Cert_Body_Issuer_RDN_DuplicateSet_Key_Value")

	def test_generated_distinguishedname_dn_issuer_rdn_empty_string(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_empty_string.pem", expect_present = "X509Cert_Body_Issuer_RDN_EmptyString")

	def test_generated_distinguishedname_dn_issuer_rdn_illegal_char(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_illegal_char.pem", expect_present = "X509Cert_Body_Issuer_RDN_IllegalCharacter")

	def test_generated_distinguishedname_dn_issuer_rdn_length_exceeded(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_length_exceeded.pem", expect_present = "X509Cert_Body_Issuer_RDN_LengthExceeded")

	def test_generated_distinguishedname_dn_issuer_rdn_multivalued(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_multivalued.pem", expect_present = "X509Cert_Body_Issuer_RDN_MultiValuedRDN")

	def test_generated_distinguishedname_dn_issuer_rdn_non_printable(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_rdn_non_printable.pem", expect_present = "X509Cert_Body_Issuer_RDN_NonPrintable")

	def test_generated_distinguishedname_dn_issuer_unusually_many_rdns(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_issuer_unusually_many_rdns.pem", expect_present = "X509Cert_Body_Issuer_UnusuallyManyRDNs")

	def test_generated_distinguishedname_dn_subject_duplicate_rdns(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_duplicate_rdns.pem", expect_present = "X509Cert_Body_Subject_DuplicateRDNs")

	def test_generated_distinguishedname_dn_subject_empty(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_empty.pem", expect_present = "X509Cert_Body_Subject_Empty")

	def test_generated_distinguishedname_dn_subject_multiple_cn(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_multiple_cn.pem", expect_present = "X509Cert_Body_Subject_MultipleCN")

	def test_generated_distinguishedname_dn_subject_no_cn(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_no_cn.pem", expect_present = "X509Cert_Body_Subject_NoCN")

	def test_generated_distinguishedname_dn_subject_rdn_deprecated_type(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_deprecated_type.pem", expect_present = "X509Cert_Body_Subject_RDN_DeprecatedType")

	def test_generated_distinguishedname_dn_subject_rdn_duplicate_key(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_duplicate_key.pem", expect_present = "X509Cert_Body_Subject_RDN_DuplicateSet_Key")

	def test_generated_distinguishedname_dn_subject_rdn_duplicate_key_value(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_duplicate_key_value.pem", expect_present = "X509Cert_Body_Subject_RDN_DuplicateSet_Key_Value")

	def test_generated_distinguishedname_dn_subject_rdn_empty_string(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_empty_string.pem", expect_present = "X509Cert_Body_Subject_RDN_EmptyString")

	def test_generated_distinguishedname_dn_subject_rdn_illegal_char(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_illegal_char.pem", expect_present = "X509Cert_Body_Subject_RDN_IllegalCharacter")

	def test_generated_distinguishedname_dn_subject_rdn_length_exceeded(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_length_exceeded.pem", expect_present = "X509Cert_Body_Subject_RDN_LengthExceeded")

	def test_generated_distinguishedname_dn_subject_rdn_multivalued(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_multivalued.pem", expect_present = "X509Cert_Body_Subject_RDN_MultiValuedRDN")

	def test_generated_distinguishedname_dn_subject_rdn_non_printable(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_rdn_non_printable.pem", expect_present = "X509Cert_Body_Subject_RDN_NonPrintable")

	def test_generated_distinguishedname_dn_subject_unusually_many_rdns(self):
		self._test_examine_x509test_resultcode("certs/generated/distinguishedname/dn_subject_unusually_many_rdns.pem", expect_present = "X509Cert_Body_Subject_UnusuallyManyRDNs")

# ====================================================================== End of distinguishedname ======================================================================



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import random
from x509sak.tests import BaseTest
from x509sak.Exceptions import InvalidInputException
from x509sak.NumberTheory import NumberTheory

class NumberTheoryTests(BaseTest):
	def test_egcd(self):
		(g, s, t) = NumberTheory.egcd(4567, 123)
		self.assertEqual(g, 1)
		self.assertEqual(s, -23)
		self.assertEqual(t, 854)

		(g, s, t) = NumberTheory.egcd(123, 4567)
		self.assertEqual(g, 1)
		self.assertEqual(s, 854)
		self.assertEqual(t, -23)

		(g, s, t) = NumberTheory.egcd(101 * 17, 101 * 11)
		self.assertEqual(g, 101)
		self.assertEqual(s, 2)
		self.assertEqual(t, -3)

	def test_modinv(self):
		p = 2003
		for _ in range(10):
			r = random.randint(2, p - 1)
			inv = NumberTheory.modinv(r, p)
			self.assertEqual(r * inv % p, 1)

	def test_gen_prime(self):
		primes = set()
		for _ in range(15):
			p = NumberTheory.randprime_bits(6, two_msb_set = True)
			q = NumberTheory.randprime_bits(6, two_msb_set = True)
			primes.add(p)
			primes.add(q)
			n = p * q
			self.assertEqual(n.bit_length(), 12)

		# With reasonal probability, all will be hit
		self.assertEqual(primes, set([ 53, 59, 61 ]))

	def test_crt(self):
		moduli = { 2: 1, 3: 2, 5: 3, 7: 4, 11: 5, 13: 6, 17: 7 }
		solution = NumberTheory.solve_crt(moduli)
		for (modulus, remainder) in moduli.items():
			self.assertEqual(solution % modulus, remainder)

	def test_pollard_rho_prime(self):
		p = 2003
		self.assertIsNone(NumberTheory.pollard_rho(p), None)

	def test_pollard_rho_success(self):
		p = 3513360553
		q = 2841366767
		n = p * q
		self.assertIn(NumberTheory.pollard_rho(n), [ p, q ])

	def test_pollard_rho_give_up(self):
		# There's a small chance this test may fail. :-)
		p = 95749913370328252241446208736506188314802113993127662348226135040475696867509
		q = 99274458486037483860318981815348157562287354834427068777154647405331367500099
		n = p * q
		self.assertIsNone(NumberTheory.pollard_rho(n, max_iterations = 10))

	def test_carryless_multiplication(self):
		for x in [ 0, 1, 2, 3, 4, 5, 123, 267, 498327849327 ]:
			self.assertEqual(NumberTheory.cl_mul(0, x), 0)
			self.assertEqual(NumberTheory.cl_mul(x, 0), 0)
			self.assertEqual(NumberTheory.cl_mul(1, x), x)
			self.assertEqual(NumberTheory.cl_mul(x, 1), x)
			self.assertEqual(NumberTheory.cl_mul(2, x), 2 * x)
			self.assertEqual(NumberTheory.cl_mul(x, 2), 2 * x)
			self.assertEqual(NumberTheory.cl_mul(x, 1024), 1024 * x)
			self.assertEqual(NumberTheory.cl_mul(1024, x), 1024 * x)

			self.assertEqual(NumberTheory.cl_mul(x, 1 | 2 | 8 | 256), (1 * x) ^ (2 * x) ^ (8 * x) ^ (256 * x))
			self.assertEqual(NumberTheory.cl_mul(1 | 2 | 8 | 256, x), (1 * x) ^ (2 * x) ^ (8 * x) ^ (256 * x))

	def test_polynomial_reduction(self):
		self.assertEqual(NumberTheory.binpoly_reduce(0, 0x10001), 0)
		self.assertEqual(NumberTheory.binpoly_reduce(0x123, 0x10001), 0x123)
		self.assertEqual(NumberTheory.binpoly_reduce(0x1234, 0x10001), 0x1234)
		self.assertEqual(NumberTheory.binpoly_reduce(0x12345, 0x10001), 0x2344)
		self.assertEqual(NumberTheory.binpoly_reduce(0x20000, 0x10001), 0x2)

	def test_sqrt_3_mod_4(self):
		p = 9937818373633759003
		self.assertEqual(p % 4, 3)

		for value in [ 1, 123, 123456789, 0x123456789 ]:
			value = value % p
			sqr = (value * value) % p
			(sqrt_pos, sqrt_neg) = NumberTheory.sqrt_mod_p(sqr, p)
			self.assertEqual((sqrt_pos & 1), 0)
			self.assertEqual((sqrt_neg & 1), 1)
			self.assertEqual((sqrt_pos * sqrt_pos) % p, sqr)
			self.assertEqual((sqrt_neg * sqrt_neg) % p, sqr)

	def test_sqrt_5_mod_8(self):
		for p in [ 11948800825345174421 ]:
			self.assertEqual(p % 8, 5)

			for value in [ 1, 123, 123456789, 0x123456789 ]:
				value = value % p
				sqr = (value * value) % p
				(sqrt_pos, sqrt_neg) = NumberTheory.sqrt_mod_p(sqr, p)
				self.assertEqual((sqrt_pos & 1), 0)
				self.assertEqual((sqrt_neg & 1), 1)
				self.assertEqual((sqrt_pos * sqrt_pos) % p, sqr)

	def test_hweight_margin(self):
		r = int.from_bytes(os.urandom(16), byteorder = "little")
		self.assertTrue(NumberTheory.hamming_weight_analysis(r).plausibly_random)

		n = 0x10000000000000
		self.assertFalse(NumberTheory.hamming_weight_analysis(n).plausibly_random)

	def test_modinv_exception(self):
		with self.assertRaises(InvalidInputException):
			NumberTheory.modinv(0, 101)

	def test_probable_prime(self):
		self.assertTrue(NumberTheory.is_probable_prime(2))
		self.assertTrue(NumberTheory.is_probable_prime(3))
		self.assertTrue(NumberTheory.is_probable_prime(5))
		self.assertTrue(NumberTheory.is_probable_prime(101))
		self.assertFalse(NumberTheory.is_probable_prime(0))
		self.assertFalse(NumberTheory.is_probable_prime(1))
		self.assertFalse(NumberTheory.is_probable_prime(4))

	def test_iterprimes(self):
		primes = [ value for (no, value) in zip(range(25), NumberTheory.iter_primes()) ]
		self.assertEqual(primes, [ 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97])

	def test_find_small_factor(self):
		self.assertEqual(NumberTheory.find_small_factor(7 * 2003), 7)
		self.assertEqual(NumberTheory.find_small_factor(2 * 2003), 2)
		self.assertEqual(NumberTheory.find_small_factor(71 * 2003), 71)
		self.assertEqual(NumberTheory.find_small_factor(2003), None)

	def test_factor(self):
		self.assertEqual(list(sorted(NumberTheory.factor(101 * 211))), [ 101, 211 ])
		self.assertEqual(list(sorted(NumberTheory.factor(2 ** 4))), [ 2, 2, 2, 2 ])
		self.assertEqual(list(sorted(NumberTheory.factor(2 * 2 * 3 * 3 * 5))), [ 2, 2, 3, 3, 5 ])

	def test_possible_divisors(self):
		self.assertEqual(list(sorted(NumberTheory.possible_divisors([ 2, 2, 3 ]))), [ 1, 2, 3, 4, 6, 12 ])
		self.assertEqual(list(sorted(NumberTheory.possible_divisors([ 2, 3, 5 ]))), [ 1, 2, 3, 5, 6, 10, 15, 30 ])

	def test_isqrt(self):
		self.assertEqual(NumberTheory.isqrt(0), 0)
		self.assertEqual(NumberTheory.isqrt(1), 1)
		self.assertEqual(NumberTheory.isqrt(2), 1)
		self.assertEqual(NumberTheory.isqrt(3), 1)
		self.assertEqual(NumberTheory.isqrt(4), 2)
		self.assertEqual(NumberTheory.isqrt(5), 2)
		self.assertEqual(NumberTheory.isqrt(27), 5)
		self.assertEqual(NumberTheory.isqrt(35), 5)
		self.assertEqual(NumberTheory.isqrt(36), 6)
		self.assertEqual(NumberTheory.isqrt(123456789), 11111)
		self.assertEqual(NumberTheory.isqrt(1234567890), 35136)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

class HexDump():
	def __init__(self):
		self._format = "full"
		self._width = 16
		self._spacers = [ 4, 8, 8 ]
		self._misschar = " "
		self._noasciichar = "."
		self._addr = True
		self._strrep = True
		assert(len(self._misschar) == 1)
		assert(len(self._noasciichar) == 1)

	def _dumpline(self, offset, data):
		line = ""

		if self._addr:
			line += "%6x   " % (offset)

		for charindex in range(self._width):
			if charindex >= len(data):
				char = self._misschar * 3
			else:
				char = " %02x" % (data[charindex])

			line += char
			for spacer in self._spacers:
				if ((charindex + 1) % spacer) == 0:
					line += " "

		if self._strrep:
			line += "|"
			for charindex in range(self._width):
				if charindex >= len(data):
					char = self._misschar
				else:
					if (32 < data[charindex] < 127):
						char = bytes([ data[charindex] ]).decode("latin1")
					else:
						char = self._noasciichar
				line += char
			line += "|"

		return line

	def as_lines(self, data):
		assert(isinstance(data, (bytes, bytearray)))
		yield from (self._dumpline(i, data[i : i + self._width]) for i in range(0, len(data), self._width))

	def as_str(self, data):
		return "\n".join(self.as_lines(data))

	def dump(self, data, fp = None):
		print(self.as_str(data), file = fp)

	def diff(self, data_left, data_right):
		for (left, right) in zip(self.as_lines(data_left), self.as_lines(data_right)):
			print("%s   %s" % (left, right))



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import json
from x509sak.OpenSSLTools import OpenSSLTools
from x509sak.PrivateKeyStorage import PrivateKeyStorage, PrivateKeyStorageForm

class CAManager():
	_FILTER_EXTENSIONS_FOR_CSR = set([ "authorityKeyIdentifier" ])
	_EXTENSION_TEMPLATES = {
		"rootca": {
			"basicConstraints":			"critical,CA:TRUE",
			"subjectKeyIdentifier":		"hash",
			"keyUsage":					"critical,digitalSignature,keyCertSign,cRLSign",
		},
		"ca": {
			"basicConstraints":			"critical,CA:TRUE",
			"subjectKeyIdentifier":		"hash",
			"authorityKeyIdentifier":	"keyid",
			"keyUsage":					"critical,digitalSignature,keyCertSign,cRLSign",
		},
		"tls-client": {
			"nsCertType":				"client",
			"basicConstraints":			"critical,CA:FALSE",
			"subjectKeyIdentifier":		"hash",
			"authorityKeyIdentifier":	"keyid",
			"keyUsage":					"critical,digitalSignature,keyAgreement,keyEncipherment",
			"extendedKeyUsage":			"clientAuth",
		},
		"tls-server": {
			"nsCertType":				"server",
			"basicConstraints":			"critical,CA:FALSE",
			"subjectKeyIdentifier":		"hash",
			"authorityKeyIdentifier":	"keyid",
			"keyUsage":					"critical,digitalSignature,keyAgreement,keyEncipherment",
			"extendedKeyUsage":			"serverAuth",
		},
	}

	def __init__(self, capath):
		self._capath = os.path.realpath(capath)
		if not self._capath.endswith("/"):
			self._capath += "/"

		if not os.path.isfile(self.metadata_filename):
			# Create if does not exist for legacy CAs.
			self._private_key_storage = PrivateKeyStorage(PrivateKeyStorageForm.PEM_FILE, filename = "CA.key", search_path = self._capath)
			self._save_metadata()
		else:
			# Load from metadata
			self._load_metadata()
		if self._private_key_storage.is_file_based:
			self._private_key_storage.update("search_path", self._capath)

	@property
	def capath(self):
		return self._capath

	def _load_metadata(self):
		with open(self.metadata_filename) as f:
			metadata = json.load(f)
		self._private_key_storage = PrivateKeyStorage.from_dict(metadata["private_key_storage"])

	def _save_metadata(self):
		metadata = {
			"version":				1,
			"private_key_storage":	self._private_key_storage.to_dict(),
		}
		with open(self.metadata_filename, "w") as f:
			json.dump(metadata, f, sort_keys = True, indent = 4)
			f.write("\n")

	def _file(self, filename):
		return self._capath + filename

	@property
	def metadata_filename(self):
		return self._file("metadata.json")

	@property
	def private_key_storage(self):
		return self._private_key_storage

	@property
	def root_crt_filename(self):
		return self._file("CA.crt")

	@property
	def serial_filename(self):
		return self._file("serial")

	@property
	def crlnumber_filename(self):
		return self._file("crlnumber")

	@property
	def crl_filename(self):
		return self._file("CA.crl")

	@property
	def index_filename(self):
		return self._file("index.txt")

	@property
	def index_attr_filename(self):
		return self._file("index.txt.attr")

	@property
	def newcerts_dirname(self):
		return self._file("certs")

	@classmethod
	def expand_extensions(cls, extension_template = None, custom_x509_extensions = None):
		extensions = { }
		if extension_template is not None:
			extensions.update(cls._EXTENSION_TEMPLATES[extension_template])
		if custom_x509_extensions is not None:
			extensions.update(custom_x509_extensions)
		return extensions

	def create_selfsigned_ca_cert(self, subject_dn, validity_days, signing_hash, serial, custom_x509_extensions = None):
		x509_extensions = self.expand_extensions(extension_template = "rootca", custom_x509_extensions = custom_x509_extensions)
		OpenSSLTools.create_selfsigned_certificate(self.private_key_storage, self.root_crt_filename, subject_dn, validity_days, signing_hash = signing_hash, serial = serial, x509_extensions = x509_extensions)

	def create_ca_csr(self, csr_filename, subject_dn):
		OpenSSLTools.create_csr(self.private_key_storage, csr_filename, subject_dn)

	@classmethod
	def create_csr(cls, private_key_storage, csr_filename, subject_dn, extension_template = None, custom_x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None):
		extensions = cls.expand_extensions(extension_template, custom_x509_extensions)
		extensions = { key: value for (key, value) in extensions.items() if key not in cls._FILTER_EXTENSIONS_FOR_CSR }
		OpenSSLTools.create_csr(private_key_storage, csr_filename, subject_dn, x509_extensions = extensions, subject_alternative_dns_names = subject_alternative_dns_names, subject_alternative_ip_addresses = subject_alternative_ip_addresses, signing_hash = signing_hash)

	def sign_csr(self, csr_filename, crt_filename, subject_dn, validity_days, extension_template = None, custom_x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None):
		extensions = self.expand_extensions(extension_template, custom_x509_extensions)
		OpenSSLTools.ca_sign_csr(self, csr_filename, crt_filename, subject_dn, validity_days, x509_extensions = extensions, subject_alternative_dns_names = subject_alternative_dns_names, subject_alternative_ip_addresses = subject_alternative_ip_addresses, signing_hash = signing_hash)

	def revoke_crt(self, *args, **kwargs):
		OpenSSLTools.ca_revoke_crt(self, *args, **kwargs)

	def create_crl(self, *args, **kwargs):
		OpenSSLTools.ca_create_crl(self, *args, **kwargs)

	def __create_management_files(self, unique_subject = True):
		with open(self.index_filename, "w") as f:
			pass
		with open(self.index_attr_filename, "w") as f:
			print("unique_subject = %s" % ("yes" if unique_subject else "no"), file = f)
		with open(self.serial_filename, "w") as f:
			print("01", file = f)
		with open(self.crlnumber_filename, "w") as f:
			print("01", file = f)
		os.mkdir(self.newcerts_dirname)

	def create_ca_structure(self, private_key_storage, unique_subject = True):
		self._private_key_storage = private_key_storage
		self._save_metadata()
		self.__create_management_files(unique_subject = unique_subject)



#!/usr/bin/python3
#
#	FriendlyArgumentParser - Argument parser with default help pages
#	Copyright (C) 2011-2012 Johannes Bauer
#
#	This file is part of pycommon.
#
#	pycommon is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	pycommon is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with pycommon; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>
#
#	File UUID c55a0ea0-6dc8-4ceb-a9ff-e54ea8a2ea62

import sys
import argparse
import textwrap

class FriendlyArgumentParser(argparse.ArgumentParser):
	def __init__(self, *args, **kwargs):
		argparse.ArgumentParser.__init__(self, *args, **kwargs)
		self.__silent_error = False

	def setsilenterror(self, silenterror):
		self.__silent_error = silenterror

	def error(self, msg):
		if self.__silent_error:
			raise Exception(msg)
		else:
			for line in textwrap.wrap("Error: %s" % (msg), subsequent_indent = "  "):
				print(line, file = sys.stderr)
			print(file = sys.stderr)
			self.print_help(file = sys.stderr)
			sys.exit(1)

def baseint(value, default_base = 10):
	if value.lower().startswith("0x"):
		return int(value, 16)
	elif value.lower().startswith("0b"):
		return int(value, 2)
	elif value.lower().startswith("0o"):
		return int(value, 8)
	elif value.lower().startswith("0b"):
		return int(value, 2)
	else:
		return int(value, default_base)

def baseint_unit(value, default_base = 10):
	units = (
		("k",	1000),
		("ki",	1024),
		("M",	1000 * 1000),
		("Mi",	1024 * 1024),
		("G",	1000 * 1000 * 1000),
		("Gi",	1024 * 1024 * 1024),
		("T",	1000 * 1000 * 1000 * 1000),
		("Ti",	1024 * 1024 * 1024 * 1024),
	)
	for (suffix, coefficient) in units:
		if value.endswith(suffix):
			return coefficient * baseint(value[:-len(suffix)], default_base = default_base)
	return baseint(value, default_base = default_base)

if __name__ == "__main__":
	parser = FriendlyArgumentParser(description = "Simple example application.")
	parser.add_argument("-d", "--dbfile", metavar = "filename", type = str, default = "mydb.sqlite", help = "Specifies database file to use. Defaults to %(default)s.")
	parser.add_argument("-f", "--force", action = "store_true", help = "Do not ask for confirmation")
	parser.add_argument("-x", metavar = "hexint", type = baseint, default = "0x100", help = "Defaults to %(default)s.")
	parser.add_argument("-v", "--verbose", action = "count", default = 0, help = "Increases verbosity. Can be specified multiple times to increase.")
	parser.add_argument("qids", metavar = "qid", type = int, nargs = "+", help = "Question ID(s) of the question(s) to be edited")
	args = parser.parse_args(sys.argv[1:])
	print(args)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2020-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.BaseAction import BaseAction
import x509sak.estimate.JudgementStructure

class ActionJudgementCode(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)

		structure = x509sak.estimate.JudgementStructure.create_judgement_structure(verbose = self._args.verbose >= 1)
		extended_enum_class = structure.create_extended_enum_class()
		if self._args.action == "list":
			codes = [ enum_code.value for enum_code in extended_enum_class ]
			for jc in sorted(codes):
				print(jc.name)
		elif self._args.action == "dump":
			structure.root.dump()
		elif self._args.action == "inherit":
			for (target_attribute, inherited_codes) in sorted(extended_enum_class.inheritance.items()):
				print("%s:" % (target_attribute))
				for (base_name, codepoint) in sorted(inherited_codes.items()):
					print("    %-60s  %s" % (base_name, codepoint.name))
				print()
		else:
			raise NotImplementedError(self._args.action)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import json
import subprocess
from x509sak.Tools import CmdTools
from x509sak.Exceptions import CmdExecutionFailedException
from x509sak.HexDump import HexDump

class ExecutionResult():
	def __init__(self, executor, stdout, stderr, return_code):
		self._executor = executor
		self._stdout = stdout
		self._stderr = stderr
		self._return_code = return_code

	@property
	def executor(self):
		return self._executor

	@property
	def stdout(self):
		return self._stdout

	@property
	def stdout_text(self):
		return self.stdout.decode(encoding = "utf-8")

	@property
	def stdout_json(self):
		return json.loads(self.stdout_text)

	@property
	def stderr(self):
		return self._stderr

	@property
	def stderr_text(self):
		return self.stderr.decode(encoding = "utf-8")

	@property
	def stdouterr(self):
		return self._stdout + self._stderr

	@property
	def stdouterr_text(self):
		return self.stdouterr.decode(encoding = "utf-8")

	@property
	def return_code(self):
		return self._return_code

	@property
	def successful(self):
		return self.return_code in self._executor.success_return_codes

	def _dump_data(self, text, bin_data):
		print("%s (%d bytes):" % (text, len(bin_data)))
		try:
			text_data = bin_data.decode("utf-8")
			print(text_data)
		except UnicodeDecodeError:
			HexDump().dump(bin_data)

	def dump(self):
		success_error_str = {
			False:	"✖",
			True:	"✓",
		}[self.successful]
		success_return_codes = ", ".join("%d" % (code) for code in sorted(self.executor.success_return_codes))
		print("%s %3d (OK = %s): %s" % (success_error_str, self.return_code, success_return_codes, self.executor.cmd_str))
		if (self.executor.stdin is None) or (len(self.executor.stdin) == 0):
			print("No stdin.")
		else:
			self._dump_data("stdin", self.executor.stdin)
		if len(self.stdout) == 0:
			print("No stdout.")
		else:
			self._dump_data("stdout", self.stdout)
		if len(self.stderr) == 0:
			print("No stderr.")
		else:
			self._dump_data("stderr", self.stderr)

class SubprocessExecutor():
	_failed_verbose = False
	_all_verbose = False
	_pause_after_failed_execution = False
	_pause_before_execution = False

	def __init__(self, cmd, success_return_codes = None, on_failure = "exception", stdin = None, env = None):
		assert(on_failure in [ "exception", "pass", "exception-nopause" ])
		assert(all(isinstance(arg, str) for arg in cmd))
		self._cmd = cmd
		self._success_return_codes = success_return_codes
		self._on_failure = on_failure
		self._stdin = stdin
		self._env = env
		if self._success_return_codes is None:
			self._success_return_codes = (0, )
		if self._env is None:
			self._env = { }

	@property
	def stdin(self):
		return self._stdin

	@property
	def success_return_codes(self):
		return self._success_return_codes

	@property
	def cmd_str(self):
		return CmdTools.cmdline(self._cmd, self._env)

	def _pre_execution(self):
		if self._all_verbose or self._pause_before_execution:
			print(self.cmd_str)
		if self._pause_before_execution:
			input("About to execute above command, press RETURN to continue...")

	def _post_execution(self, execution_result):
		dumped = False
		if self._all_verbose or (self._failed_verbose and (not execution_result.successful) and (self._on_failure != "pass")):
			dumped = True
			execution_result.dump()

		# Execution failed.
		if not execution_result.successful:
			if self._on_failure == "exception":
				if self._pause_after_failed_execution:
					if not dumped:
						# Don't output this twice.
						execution_result.dump()
					input("Hit ENTER to continue...")
			if self._on_failure in [ "exception", "exception-nopause" ]:
				raise CmdExecutionFailedException("Execution of subprocess failed: %s" % (os.path.basename(self._cmd[0])), execution_result = execution_result)

	def run(self):
		self._pre_execution()
		env = dict(os.environ)
		env.update(self._env)

		proc = subprocess.Popen(self._cmd, stdout = subprocess.PIPE, stderr = subprocess.PIPE, stdin = subprocess.PIPE, env = env)
		(stdout, stderr) = proc.communicate(self._stdin)
		execution_result = ExecutionResult(executor = self, stdout = stdout, stderr = stderr, return_code = proc.returncode)

		self._post_execution(execution_result)
		return execution_result

	@classmethod
	def set_failed_verbose(cls):
		cls._failed_verbose = True

	@classmethod
	def set_all_verbose(cls):
		cls._all_verbose = True

	@classmethod
	def pause_after_failed_execution(cls):
		cls._pause_after_failed_execution = True

	@classmethod
	def pause_before_execution(cls):
		cls._pause_before_execution = True



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.PrivateKeyStorage import PrivateKeyStorageForm
from x509sak.Tools import PathTools

class OpenSSLConfig():
	def __init__(self):
		self._engine_dynamic_path = None
		self._engine_module_path = None
		self._x509_extensions = { }
		self._subject_alternative_dns_names = [ ]
		self._subject_alternative_ip_addresses = [ ]

	@property
	def extension_count(self):
		return len(self._x509_extensions) + len(self._subject_alternative_dns_names) + len(self._subject_alternative_ip_addresses)

	def dump(self):
		print("OpenSSL config:")
		print("    Engine dynamic path : %s" % (self._engine_dynamic_path))
		print("    Engine MODULE path  : %s" % (self._engine_module_path))
		print("    X.509 extensions    : %s" % (str(self._x509_extensions)))
		print("    Subj alternative DNS: %s" % (str(self._subject_alternative_dns_names)))
		print("    Subj alternative IP : %s" % (str(self._subject_alternative_ip_addresses)))

	@property
	def use_engine(self):
		return self._engine_dynamic_path is not None

	def set_private_key_storage(self, private_key_storage):
		if private_key_storage.storage_form == PrivateKeyStorageForm.HARDWARE_TOKEN:
			self._engine_dynamic_path = PathTools.find(private_key_storage.so_search_path, private_key_storage.dynamic_so)
			self._engine_module_path = PathTools.find(private_key_storage.so_search_path, private_key_storage.module_so)

	def set_x509_extensions(self, x509_extensions):
		if x509_extensions is None:
			x509_extensions = { }
		self._x509_extensions = x509_extensions

	def set_subject_alternative_dns_names(self, subject_alternative_dns_names):
		if subject_alternative_dns_names is None:
			subject_alternative_dns_names = [ ]
		self._subject_alternative_dns_names = subject_alternative_dns_names

	def set_subject_alternative_ip_addresses(self, subject_alternative_ip_addresses):
		if subject_alternative_ip_addresses is None:
			subject_alternative_ip_addresses = [ ]
		self._subject_alternative_ip_addresses = subject_alternative_ip_addresses


	def __write_config(self, f):
		print("openssl_conf = openssl_conf", file = f)
		print("oid_section = oids", file = f)
		print(file = f)
		print("[openssl_conf]", file = f)
		if self.use_engine:
			print("engines = engines_config", file = f)
		print(file = f)
		print("[engines_config]", file = f)
		print("pkcs11 = engine_pkcs11_parameters", file = f)
		print(file = f)
		print("[engine_pkcs11_parameters]", file = f)
		print("engine_id = pkcs11", file = f)
		print("dynamic_path = %s" % (self._engine_dynamic_path), file = f)
		print("MODULE_PATH = %s" % (self._engine_module_path), file = f)
		print("init = 0", file = f)
		print(file = f)
		print("[ca]", file = f)
		print("default_ca = CA_default", file = f)
		print(file = f)
		print("[CA_default]", file = f)
		print("database = index.txt", file = f)
		print("new_certs_dir = certs", file = f)
		print("certificate = CA.crt", file = f)
		print("private_key = CA.key", file = f)
		print("serial = serial", file = f)
		print("crlnumber = crlnumber", file = f)
		print("crl = CA.crl", file = f)
		print("default_md = sha256", file = f)
		print("default_days = 365", file = f)
		print("default_crl_days = 30", file = f)
		print("policy = policy_onlyCN", file = f)
		if self.extension_count > 0:
			print("x509_extensions = extensions", file = f)
		print(file = f)
		print("[policy_onlyCN]", file = f)
		print("commonName = supplied", file = f)
		print("surname = optional", file = f)
		print("serialNumber = optional", file = f)
		print("countryName = optional", file = f)
		print("localityName = optional", file = f)
		print("stateOrProvinceName = optional", file = f)
		print("streetAddress = optional", file = f)
		print("organizationName = optional", file = f)
		print("organizationalUnitName = optional", file = f)
		print("title = optional", file = f)
		print("description = optional", file = f)
		print("searchGuide = optional", file = f)
		print("businessCategory = optional", file = f)
		print("postalAddress = optional", file = f)
		print("postalCode = optional", file = f)
		print("postOfficeBox = optional", file = f)
		print("telephoneNumber = optional", file = f)
		print("name = optional", file = f)
		print("givenName = optional", file = f)
		print("pseudonym = optional", file = f)
		print("emailAddress = optional", file = f)
#		print("1.2.3.4 = optional", file = f)
#		print("myfield = optional", file = f)
		print(file = f)
		print("[req]", file = f)
		print("distinguished_name = default_dn", file = f)
		if self.extension_count > 0:
			print("x509_extensions = extensions", file = f)
			print("req_extensions = extensions", file = f)
		print(file = f)
		print("[oids]", file = f)
#		print("myfield = 1.2.3.4", file = f)
		print(file = f)
		print("[default_dn]", file = f)
		print("myfield = Custom field myfield", file = f)
		print("myfield_default =", file = f)
		print(file = f)
		print("[extensions]", file = f)
		for (key, value) in sorted(self._x509_extensions.items()):
			print("%s = %s" % (key, value), file = f)

		alt_names = [ ]
		alt_names += [ "DNS:%s" % (value) for value in self._subject_alternative_dns_names ]
		alt_names += [ "IP:%s" % (value) for value in self._subject_alternative_ip_addresses ]
		if len(alt_names) > 0:
			print("subjectAltName = %s" % (", ".join(alt_names)), file = f)

	def write_to(self, filename):
		with open(filename, "w") as f:
			self.__write_config(f)
